
// SLMediaInfoTestDlg.h : header file
//

#pragma once
#include "afxwin.h"
#include <string>
#include <sstream>
#include <iomanip>
#include <vector>
#include <set>
#include <chrono>
#include <atlsafe.h>

DEFINE_AUTOMATION_TYPE_FUNCTION(MediaInfoEx, MediaInfoEx, VT_RECORD);

// CSLMediaInfoTestDlg dialog
class CSLMediaInfoTestDlg : public CDialog
{
// Construction
public:
	CSLMediaInfoTestDlg(CWnd* pParent = NULL);	// standard constructor

// Dialog Data
	enum { IDD = IDD_SLMEDIAINFOTEST_DIALOG };

	protected:
	virtual void DoDataExchange(CDataExchange* pDX);	// DDX/DDV support


// Implementation
protected:
	HICON m_hIcon;
    ISLMediaInfoCreatorPtr m_mediaInfoCreator;

	// Generated message map functions
	virtual BOOL OnInitDialog();
	afx_msg void OnSysCommand(UINT nID, LPARAM lParam);
	afx_msg void OnPaint();
	afx_msg HCURSOR OnQueryDragIcon();
	DECLARE_MESSAGE_MAP()
public:
    static CString GetScreenAspectStr(MediaInfo* info, bool wide);
    static void CalcScreenAspect(MediaInfo* info, long* screenAspectX, long* screenAspectY);
    static void CalcPixelAspect(MediaInfo* info, long* pixelAspectX, long* pixelAspectY);
    static CString GetScreenAspectStr(long cx, long cy, long srcAX, long srcAY, bool wide);
    static void CalcScreenAspect(long cx, long cy, long srcAX, long srcAY, long* screenAspectX, long* screenAspectY);
    static void CalcPixelAspect(long cx, long cy, long srcAX, long srcAY, long* pixelAspectX, long* pixelAspectY);
    void UpdateMediaInfo(MediaInfo* info);
    void UpdateMediaInfo(ISLMediaInfo* iMediaInfo);
    static LPCTSTR GetMediaTypeStr(SLMediaType type) {
        switch (type) {
        case SLMediaType_VideoAndAudio:  return _T("VideoAndAudio");
        case SLMediaType_AllAudio:            return _T("AllAudio");
        case SLMediaType_AllImages:          return _T("AllImages");
        //case SLMediaType_Film:                    return _T("Film");
        case SLMediaType_AllMovies:          return _T("AllMovies");
        case SLMediaType_All:                      return _T("All");
        //case SLMediaType_Pause:                  return _T("Pause");
        case SLMediaType_Unknown:              return _T("Unknown");
        case SLMediaType_Image:                  return _T("Image");
        case SLMediaType_Avi:                      return _T("Avi");
        case SLMediaType_Mpeg2:                  return _T("Mpeg2");
        case SLMediaType_ASF:                      return _T("ASF");
        case SLMediaType_QT:                        return _T("QT");
        case SLMediaType_Mxf:                      return _T("Mxf");
        case SLMediaType_FFMpeg:                return _T("FFMpeg");
        case SLMediaType_Wav:                      return _T("Wav");
        //case SLMediaType_Count:                  
        default:
            return _T("Unk");
        }
    }

    std::wstring ToString(const MediaInfoEx& iMediaInfo) {
        using namespace std;
        auto os = wostringstream();
        os 
            << setw(6) << GetMediaTypeStr(iMediaInfo._type) 
            << " v:" << iMediaInfo._srcVideoPresent
            << " a:" << iMediaInfo._srcAudioPresent
            << " " << left << setw(12) << (wostringstream() << "(" << iMediaInfo._srcCX << "x" << iMediaInfo._srcCY << ")").str()
            << " " << right << setw(6) /*<< setprecision(5)*/ << iMediaInfo._durationSec << " sec"
            << " '" << (LPCWSTR)iMediaInfo._name << "'";

        return os.str();
    }
    std::wstring ToString(const MediaInfo& iMediaInfo, LPCTSTR name, double time) {
        using namespace std;
        auto os = wostringstream();
        os
            << right << setw(6) << setprecision(5) << time << " "
            << setw(6) << GetMediaTypeStr(iMediaInfo._type)
            << " v:" << iMediaInfo._srcVideoPresent
            << " a:" << iMediaInfo._srcAudioPresent
            << " " << left << setw(12) << (wostringstream() << "(" << iMediaInfo._srcCX << "x" << iMediaInfo._srcCY << ")").str()
            << " " << right << setw(6) /*<< setprecision(5)*/ << iMediaInfo._durationSec << " sec"
            << " '" << name << "'";

        return os.str();
    }
    //////////////////////////////////////////////////////////////////////////
    //   
    static inline long gcd(long a, long b) {
        while (b != 0) {
            long c = a % b;
            a = b;
            b = c;
        }
        return abs(a);
    }

    CString m_mediaFile;
    afx_msg void OnBnClickedBrowse();
    double m_duration;
    afx_msg void OnDestroy();
    afx_msg void OnBnClickedCheck();
    void SetCheckTime(double time, int nID);
    CComboBox m_comboAudioStreams;
    afx_msg void OnBnClickedOk();
    afx_msg void OnBnClickedCancel();
    CComboBox m_comboSatellites;
    CListBox m_listFiles;
    afx_msg
        void YieldMessage();
    void OnBnClickedCheckDir();

    void FindMediaFiles(CPath path, CSimpleArray<CString> &extArr, CAtlArray<CString> &mediaFiles, bool withSubdirs);
    void GetExtFromFilter(const CString& filter, CSimpleArray<CString>& extArray);

    std::vector<CString> FindMediaFiles(CPath path, const std::set<CString>& extensions, bool withSubdirs);
    std::set<CString> GetExtFromFilter(const CString& filter);

    std::vector<CString> FindMediaFiles(CPath path, bool withSubdirs);

    template <class T>
    static void RandomizeMediaFiles(std::vector<T>& mediaFiles);
    static std::vector<MediaInfoEx> RandomizeMediaInfo(const CComSafeArray<MediaInfoEx, VT_RECORD>& mediaFiles);

    CListBox m_listErrors;
    BOOL m_withSubdirs = FALSE;
    CSize m_minSize{ 0, 0 };
	afx_msg void OnGetMinMaxInfo(MINMAXINFO* lpMMI);
    afx_msg void OnBnClickedCheckDirEx();
    BOOL m_fast = TRUE;
    BOOL m_fastFindFile = TRUE;
    int m_InAction = 0;
    struct ReenterLock {
        int& _inAction;
        ReenterLock(int& inAction) :_inAction(inAction) {
            _inAction++;
        }
        ~ReenterLock() {
            _inAction--;
        }
        operator bool() { return _inAction > 1; }
    };
    afx_msg void OnBnClickedPickTime();
    std::vector<CString> GetFiles();
    CSpinButtonCtrl m_spinTime;
    BOOL m_shuffle{ TRUE };
    
    template <typename duration>
    static CString toSMPTEmSecString(duration ts) {
        //if (ts.count() == no_pts_value)
        //    return _T("nopts");
        //while (ts >= mpeg_wrap)
        //    ts -= mpeg_wrap;

        int sign = ts.count() < 0 ? -1 : 1;
        ts *= sign;
        auto h = std::chrono::duration_cast<std::chrono::hours>(ts);
        auto m_t = std::chrono::duration_cast<std::chrono::minutes>(ts);
        auto s_t = std::chrono::duration_cast<std::chrono::seconds>(ts);

        auto m = (int)(m_t - std::chrono::duration_cast<std::chrono::minutes>(h)).count();
        auto s = (int)(s_t - std::chrono::duration_cast<std::chrono::seconds>(m_t)).count();
        auto ms = (int)(std::chrono::duration_cast<std::chrono::milliseconds>(ts) - std::chrono::duration_cast<std::chrono::milliseconds>(s_t)).count();

        CString str;
        str.Format(_T("%s%02d:%02d:%02d.%03d"), sign < 0 ? _T("-") : _T(" "), (int)h.count(), m, s, ms);
        return str;
    }

    afx_msg void OnBnClickedScanStart();
    afx_msg void OnBnClickedScanStop();
    afx_msg void OnBnClickedScanAdd();
    afx_msg void OnBnClickedScanRemove();
    void UpdateScanningState(ISLMediaInfoConfigurator* configurator) {
        if (configurator != nullptr)
            SetDlgItemText(IDC_SCAN_STATE, configurator->Running ? _T("Running") : _T("Stopped"));
    }
    afx_msg void OnBnClickedPickTimeEx();
};
